##-----------------------------------------------------------------------------------
## eOn is free software: you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation, either version 3 of the License, or
## (at your option) any later version.
##
## A copy of the GNU General Public License is available at
## http://www.gnu.org/licenses/
##-----------------------------------------------------------------------------------

# Authors: Ian Johnson, Rye Terrell

import ConfigParser
import numpy
import os.path
import sys
import string
import config
import yaml

config.init_done = False

config.format = []

class ConfigSection:
    def __init__(self, name):
        self.name = name
        self.keys = []
class ConfigKey:
    def __init__(self, name, kind, default):
        self.name = name
        self.kind = kind
        self.default = default
        self.values = []
try:
    y = yaml.load(open(os.path.join(sys.path[0], "config.yaml"), 'r'))
except IOError:
    y = yaml.load(open(os.path.join(sys.path[0],"include","config.yaml"),'r'))

for sectionName in y:
    section = ConfigSection(sectionName)
    for key in y[sectionName]['options']:
        kattr = y[sectionName]['options'][key]
        ck = ConfigKey(key, kattr['kind'], kattr['default'])
        section.keys.append(ck)
        if 'values' in kattr:
            for value in kattr['values']:
                ck.values.append(value)
    config.format.append(section)
    
def init(config_file = ""):
    if config.init_done:
        return None
    config.init_done = True

    parser = ConfigParser.SafeConfigParser()
    
    for i in range(len(config.format)):
        parser.add_section(config.format[i].name)
        for j in config.format[i].keys:
            parser.set(config.format[i].name, j.name,str (j.default))

    gave_config = True
    if config_file != "":
        if os.path.isfile(config_file):
            parser.read(config_file)
            config.config_path = os.path.abspath(config_file)
        else:
            print >> sys.stderr, "specified configuration file %s does not exist" % config_file
            sys.exit(2)
    elif os.path.isfile('config.ini'):
        parser.read('config.ini')
        config.config_path = os.path.abspath('config.ini')
        gave_config = False
    else:
        print >> sys.stderr, "You must provide a configuration file either by providing it as a command line argument or by placing a config.ini in the current directory."
        sys.exit(2)        
    sections = False
    options = False

    # makes sections all lowercase
    psections = parser.sections()
    fsections = [j.name for j in config.format]
    for a in range(len(psections)):
        psections[a] = psections[a].lower()
    for a in range(len(fsections)):
        fsections[a] = fsections[a].lower()
    
    # checks all sections in config.ini are in configparser 
    config_error = False
    for s in psections:
        if s not in fsections:
            config_error = True
            sys.stderr.write('unknown section "%s"\n')

    # checks all options in config.ini are in configparser
    for i in parser.sections():
        b = parser.options(i)
        for a in range(len(b)):
            b[a] = b[a].lower()
        for k in config.format:
            if k.name == i:
                foptions = [j.name for j in k.keys]
                for a in range(len(foptions)):
                    foptions[a] = foptions[a].lower()
                for o in b:
                    if o not in foptions:
                        config_error = True
                        sys.stderr.write('unknown option "%s" in section "%s"\n' % (o, k.name))
    
    # checks all options are the right type and if parser option has values config.ini is using one of those values  
    for psection in parser.sections():
        poptions = parser.options(psection)
        for fsection in config.format:
            if psection == fsection.name:
                for k in fsection.keys:
                    for o in poptions:
                        if o == k.name:
                            if k.kind == "int":
                                try:
                                    x = int(parser.get(psection,k.name))
                                except:
                                    config_error = True
                                    sys.stderr.write('option "%s" in section "%s" should be an integer\n' % (o,psection))
                            elif k.kind == "float":
                                try:
                                    x = float(parser.get(psection, k.name))
                                except:
                                    config_error = True
                                    sys.stderr.write('option "%s" of section "%s" should be a float\n' %(o,psection))
                            elif k.kind == "boolean":
                                booleans = ['True', 'true', 'T', 't', '0', 'False', 'false', 'F', 'f', '1']
                                if parser.get(psection,k.name) not in booleans:
                                    config_error = True
                                    sys.stderr.write('option "%s" of section "%s" should be boolean\n' %(o,psection))
                            elif k.kind == "string" and len(k.values) !=0:   
                                values = k.values
                                if parser.get(psection,k.name) not in values:
                                    Vnames = ", ".join(k.values)
                                    config_error = True
                                    sys.stderr.write('option "%s" should be one of: %s\n' %(parser.get(psection,k.name),Vnames))

    if config_error:
        sys.stderr.write("aborting: could not parse config.ini\n")
        sys.exit(1)

    #Main options
    config.main_job = parser.get('Main', 'job')
    config.main_temperature = parser.getfloat('Main', 'temperature')
    config.main_random_seed = parser.getint('Main', 'random_seed')
    if int(config.main_random_seed) >= 0:
        numpy.random.seed(config.main_random_seed)
    else:
        config.main_random_seed = None
            
    #KMC options
    config.max_kmc_steps                = parser.getint('KMC', 'max_kmc_steps')
    config.kmc_max_steps_onerun         = parser.getint('KMC','max_steps_onerun')
    config.akmc_essential_dynamics      = parser.getboolean('KMC', "essential_dynamics")
    config.kmc_write_dynamics           = parser.getboolean('KMC','write_dynamics')
    config.kmc_initial_state            = parser.getint('KMC','initial_state')
    config.kmc_initial_random_boltzmann = parser.getboolean('KMC','initial_random_boltzmann')
    config.kmc_restart_new_state        = parser.getboolean('KMC','restart_new_state')
    config.kmc_track_visited_states     = parser.getboolean('KMC','track_visited_states')
    config.kmc_track_residence_times    = parser.getboolean('KMC','track_residence_times')
    config.kmc_confidence_level         = parser.getfloat('KMC','confidence_level')
    config.kmc_calc_confidence          = parser.getboolean('KMC','calc_confidence')
    config.kmc_max_thermal_window       = parser.getfloat('KMC','max_thermal_window')
    config.kmc_read_initial_statelist   = parser.getboolean('KMC','read_initial_statelist')
    config.kmc_calc_binding_energy      = parser.getboolean('KMC','calc_binding_energy')
    config.kmc_substrate_energy         = parser.getfloat('KMC','substrate_energy')


    #porefind options
    config.porefind_be_start            = parser.getfloat('Porefind','be_start')
    config.porefind_be_end              = parser.getfloat('Porefind','be_end')

    #path options
    config.path_root         = os.path.abspath(os.path.expanduser( parser.get('Paths', 'main_directory') ) )
    config.path_states       = os.path.expanduser( parser.get('Paths', 'states') )
    config.path_trajectory   = os.path.join(config.path_root, parser.get('Paths','trajectory') )
    config.path_dynamics     = os.path.join(config.path_root, parser.get('Paths', 'dynamics') )
    config.path_visited_states = os.path.join(config.path_root,parser.get('Paths', 'visited_states') )
    config.path_visited_positions = os.path.join(config.path_root,parser.get('Paths','visited_positions') )
    config.path_randomstate  = os.path.join(config.path_root,parser.get('Paths', 'randomstate') )

    #diffusion options
    config.diffusion_calc               = parser.getboolean('Diffusion','calc_diffusion')
    config.diffusion_addmolid           = parser.getint('Diffusion','addmolid')
    config.diffusion_delta_t            = parser.getfloat('Diffusion','delta_t')
    config.diffusion_delta_t0           = parser.getfloat('Diffusion','delta_t0')
    config.diffusion_max_t              = parser.getfloat('Diffusion','max_t')
    config.diffusion_calc_trajectory    = parser.getboolean('Diffusion','calc_trajectory')
    config.diffusion_evaluate_substrates= parser.getboolean('Diffusion','evaluate_substrates')
    config.diffusion_read_substrates     = parser.getboolean('Diffusion','read_substrates')
    config.diffusion_trans_cutoff       = parser.getfloat('Diffusion','trans_cutoff')
    config.diffusion_angle_cutoff       = parser.getfloat('Diffusion','angle_cutoff') 
    config.diffusion_trajectory_frequency= parser.getint('Diffusion','trajectory_frequency')
             
      
    #Coarse Graining
    config.sb_on = parser.getboolean('Coarse Graining', 'use_projective_dynamics')
    config.sb_state_file = parser.get('Coarse Graining', 'state_file') 
    config.sb_dynamic = parser.getboolean('Coarse Graining', 'dynamic_superbasins')
    config.sb_max_repeats = parser.getint('Coarse Graining', 'max_repeats')
    config.sb_path = None
    if config.sb_on:        
        config.sb_max_size = parser.getint('Coarse Graining', 'max_size')
        config.sb_path = parser.get('Paths', 'superbasins')
        config.sb_scheme = parser.get('Coarse Graining', 'superbasin_scheme')
        if config.sb_scheme == 'transition_counting':
            config.sb_tc_ntrans = parser.getint('Coarse Graining', 'number_of_transitions')
        elif config.sb_scheme == 'energy_level':
            config.sb_el_energy_increment = parser.getfloat('Coarse Graining', 'energy_increment')
        

    config.askmc_on = parser.getboolean('Coarse Graining','use_askmc')
    if config.askmc_on:
        config.askmc_confidence = parser.getfloat('Coarse Graining','askmc_confidence')
        config.askmc_alpha = parser.getfloat('Coarse Graining','askmc_barrier_raise_param')
        config.askmc_gamma = parser.getfloat('Coarse Graining','askmc_high_barrier_def')
        config.askmc_barrier_test_on = parser.getboolean('Coarse Graining','askmc_barrier_test_on')
        config.askmc_connections_test_on = parser.getboolean('Coarse Graining','askmc_connections_test_on')
        

    #Debug options
    config.debug_interactive_shell = parser.getboolean('Debug', 'interactive_shell')
    if config.debug_interactive_shell:
        import signal, code
        signal.signal(signal.SIGQUIT, lambda signum, frame: code.interact(local=locals()))
    config.debug_keep_bad_saddles  = parser.getboolean('Debug', 'keep_bad_saddles')
    config.debug_keep_all_results  = parser.getboolean('Debug', 'keep_all_result_files')
    config.debug_results_path = parser.get('Debug', 'result_files_path')
    config.debug_register_extra_results = parser.getboolean('Debug', 'register_extra_results')
    config.debug_use_mean_time = parser.getboolean('Debug', 'use_mean_time')
    config.debug_target_trajectory = parser.get('Debug', 'target_trajectory')
    config.debug_always_confident = parser.getboolean('Debug','always_confident')    
    config.debug_store_superbasins = parser.getboolean('Debug','store_superbasins')

    del parser
