#!/usr/bin/python
import numpy
import config
import sys
from eon_extract import _fit_box
import logging
logger = logging.getLogger('residence')
import eon_read

class residenceTimes:

    def __init__(self, statelist):
        self.statelist = statelist
        self.numstates = len(self.statelist)
        
        
        if config.sb_on:
            self.num_superbasins = 0
            self.box = statelist[0]['reactant_config'].box

        #initialize data for states
        self.data = {}
        stateid = 0
        while stateid < self.numstates:
            statedict = {   'num_visits': 0,
                            'time': 0.0,
                            'states': [ stateid ]
                        }
            if config.diffusion_calc:
                statedict['com'] = self.statelist[stateid]['com']                                
            self.data[stateid]=statedict
            stateid+=1

    def register(self,delta_t,stateid,superbasin=None):
        #step from a superbasin
        if superbasin:
            stateid = superbasin.id + self.numstates
            if stateid not in self.data:
                self._appendsb(stateid,superbasin)        

        self.data[stateid]['num_visits']+=1
        self.data[stateid]['time'] += delta_t
        
    def write_data(self):
        #first write visited positions
        outfile = eon_read._open_file_writing(config.path_visited_positions,'w')
        outfile.write("#Columns:\n#1: X\n#2: Y\n#3: Z\n#4: Time spent\n#5: Number of visits\n#6: (sb)-stateid\n#7: Number of states\n#8: Contained states\n")
        for stateid in sorted( self.data.keys()):
            d= self.data[stateid]
            outfile.write("%f\t%f\t%f\t%e\t%d\t%d\t%d\t%s\n" % (d['com'][0],d['com'][1],d['com'][2],
                                                        d['time'], d['num_visits'],stateid,len(d['states']),
                                                        repr(d['states'])
                                                        ) )
        outfile.close()                                                
                                                        
        #distribute residence times over real states.
        for stateid in sorted( self.data.keys() ):
            d = self.data[stateid]
            if len(d['states']) > 1:
                residencetime = d['time'] / len(d['states'])
                numvisited = float( d['num_visits'] ) / len(d['states'])
            
                for realstateid in d['states']:
                    self.data[realstateid]['time']+=residencetime
                    self.data[realstateid]['num_visits'] +=numvisited
                                                 
        outfile = eon_read._open_file_writing(config.path_visited_states,'w')   
        outfile.write("#Columns:\n#1: stateid\n#2: average residence time\n#3: number of visits\n")
        stateid = 0
        while stateid < self.numstates:
            d = self.data[stateid]
            if d['num_visits'] !=0:
                outfile.write("%d\t%e\t%f\n" % ( stateid, d['time']/d['num_visits'], d['num_visits'] ) )
            stateid +=1
            
        outfile.close()            
    
    def _appendsb(self,stateid,superbasin):
        #calculate the center of mass
        numstates = len(superbasin.states)
        states = superbasin.states[:]
        com = numpy.zeros(3)
        istate = 1

        while istate < numstates:
            com += _fit_box( self.statelist[ states[istate]]['com'] - self.statelist[ states[0] ]['com'], self.box)
            istate +=1

        com = com*(1.0/numstates)
        com += self.statelist[ states[0] ]['com']
        for i in range(3):
            if (com[i] > self.box[i][i]):
                com[i] -= self.box[i][i]
            elif (com[i] < 0.0):
                com[i] += self.box[i][i]   

        self.data[stateid] = {   'com': com,
                                 'time': 0.0,
                                 'num_visits': 0,
                                 'states': states
                             }
        return
