#!/usr/bin/python
import eon_read
import eon_extract
import sys
import os
#################################################################################################################################

def compare_substrates(config1,config2,addmolid,trans_cutoff,angle_cutoff):

    retval = True
    
    movement = eon_extract.molecule_movement(config1,config2,trans_cutoff,angle_cutoff)
    for movedmol in movement:
        if movedmol['molid'] !=addmolid:
            sys.stderr.write("Comparison failed due to mol %s with atoms %s\n" % (movedmol,config1.molecules[movedmol['molid']]._atomids))
            return False
    return True

#################################################################################################################################

def determine_substrates(statelist,addmolid,trans_cutoff = 0.5, angle_cutoff=40):
    substrate = []
    blueprints = { }
    for state in statelist:
        reactant = state['reactant_config']
        print "determining substrate for state %d" % state['stateid']
        eon_read.read_molecules(reactant,read_com = True, read_orientation = True)    
        
        #compare to known substrates:
        foundsubstrate = False
        revsubstratelist = range(len(substrate))
        revsubstratelist.reverse()
        for substrateid in revsubstratelist:
            if compare_substrates(reactant, blueprints[substrateid], addmolid, trans_cutoff, angle_cutoff )==True:
                foundsubstrate=True
                substrate[substrateid].append(state['stateid'])
                #sys.stderr.write("Added state %d to substrate %d\n" % (state['stateid'],substrateid))
                state['substrateid'] = substrateid
                break
        #if no match is found, it must be a new substrate
        if foundsubstrate==False:
            substrate.append([])
            substrateid = len(substrate)-1
            substrate[substrateid].append(state['stateid'])
            blueprints[substrateid] = reactant
            state['substrateid'] = substrateid
            #sys.stderr.write("Found new substrate(id %d), state %d\n" % (substrateid,state['stateid']))

    #delete molecule information
    for state in statelist:
        del state['reactant_config'].molecules

    return substrate

#################################################################################################################################

def read_substrates(statelist, substratepath):
    '''
    Function tries to read files called 'substrate_%d.dat' from directory
    '''
    substrates = []
    
    for filename in os.listdir(substratepath):
        print "filename=%s" % filename
        if filename.split('_')[0] == 'substrate':
            substrateid = int( filename.split('_')[1].split('.')[0] )
            substratefile = eon_read._open_file_reading(os.path.join(substratepath,filename))
            line = substratefile.readline().split()
            stateids = [int(i) for i in line ]
            substrates.append(stateids)
            for stateid in stateids:
                statelist[stateid]['substrateid'] = substrateid

    for state in statelist:
        if 'substrateid' not in state.keys():           
            state['substrateid'] = -1
    return substrates            
